import sys
import json
from sparrow.src.commands import global_flags, commands
from sparrow.src.helper import general_help, command_not_found
from sparrow.src import __version__
from sparrow.src.manager import SparrowManager


class Parser:
    """
    Parser responsibility is to direct the execution to the right
    channel.
    """

    def __init__(self):
        self.args = sys.argv[1:]
        if len(self.args) == 0:
            print(general_help())
        else:
            self.validate(self.args)

    def validate(self, cmd):
        """
        general validation mechanism for all
        commands and global flags
        :param cmd:
        :return: global_flags or commands
        """
        _global = global_flags.key
        if cmd[0] in _global['version']['key'] or cmd[0] in _global['help']['key']:
            return self.global_flags()

        if cmd[0] in [commands.key['extract']['key'], commands.key['info']['key']]:
            return self.commands()

        print(general_help(gray=True))
        print(command_not_found(cmd))

    def global_flags(self):
        """
        :return: None
        """
        if self.args[0] in ['--version', '-v'] and len(self.args) is 1:
            print("sparrow version:", __version__)
        elif self.args[0] in ['--help', '-h'] and len(self.args) is 1:
            print(general_help())
        else:
            print(general_help(gray=True))
            print(command_not_found(self.args))

    def commands(self):
        """
        commands maps to SparrowManager a valid json files
        :return:
        """
        _sparrow = SparrowManager()
        if self.args[0] == "extract" and len(self.args) == 3:
            inp = self.args[1] if self.args[1].find(".") != -1 else self.args[1] + ".json"
            tar = self.args[2] if self.args[2].find(".") != -1 else self.args[2] + ".json"
            try:
                with open(inp, 'r') as f:
                    source = json.load(f)
            except FileNotFoundError as e:
                print(e)
                exit(1)

            try:
                with open(tar, 'r') as f:
                    target = json.load(f)
            except FileNotFoundError as e:
                print(e)
                exit(1)
            return _sparrow.extract(source, target)
        elif self.args[0] == "info" and len(self.args) == 2:
            inp = self.args[1] if self.args[1].find(".") != -1 else self.args[1] + ".json"
            try:
                with open(inp, 'r', encoding='utf8') as f:
                    source = json.load(f)
            except FileNotFoundError as e:
                print(e)
                exit(1)
            return _sparrow.info(source)
        else:
            if len(self.args) == 1:
                print(general_help())
            else:
                print(general_help(gray=True))
                print(command_not_found(self.args[1:]))



"""
config is the entry point to add, edit and remove commands
and their help to extend sparrow-cli

All commands and flags are defined in the config module.
"""
from collections import namedtuple

_args = namedtuple('commands', "key help")

global_flags = _args(
    {
        'version': {'key': ['-v', '--version'], 'help': "output the version number"},
        'help'   : {'key': ['-h', '--help'], 'help': "output help information"},
    }
    , "")

commands = _args(
    {
        'extract': {'key': 'extract', 'help': "extract configuration file"},
        'info'   : {'key': 'info', 'help': "print information about the container"},
    }
    , ""
)




from sparrow.src.helper import info, extract_info
import json


class SparrowManager:

    @staticmethod
    def extract(source, target, fname="output.json"):

        target_key = [k for k in target.keys()][0]

        temp_tags = []
        temp_triggers_ids = []
        temp_triggers = []
        not_found_tags = []

        for stp in source.get('containerVersion').get('tag'):
            if stp.get('setupTag'):
                for s in stp.get('setupTag'):
                    target[target_key].append(s['tagName'])

        for k in target[target_key]:
            for t in source['containerVersion']['tag']:
                if t['name'] == k:
                    temp_tags.append(t)
                    try:
                        temp_triggers_ids.append(t['firingTriggerId'])
                    except KeyError:
                        pass
                else:
                    not_found_tags.append(t['name'])

        for tri in source['containerVersion']['trigger']:
            for t in extract_info(temp_triggers_ids):
                if t == tri['triggerId']:
                    temp_triggers.append(tri)

        source['containerVersion']['tag'] = temp_tags
        source['containerVersion']['trigger'] = temp_triggers

        with open('output.json', "w", encoding='utf8') as f:
            json.dump(fp=f, obj=source, ensure_ascii=False)
            print("Successful > output.json generated")
        SparrowManager.info(source)

    @staticmethod
    def info(source):
        print(info(source))



def extract_info(src: list or dict, what="trigger", ids=True) -> list:
    """
    extract_info extracts variables and tags ids recursively from dict
    extract_info can operate on list or nested lists as well

    :param ids:
    :param src: accepts lists or dict
    :param what: accepts 3 options trigger, tag and variable default to trigger
    :return: list: output a list of unique ids
    """
    i = []
    o = []

    if type(src) is dict:
        t = 'tagId' if ids else 'name'
        tr = 'firingTriggerId' if ids else 'name'
        v = 'variableId' if ids else 'name'
        if what == "tag":
            for tag in src.get('containerVersion').get('tag'):
                if tag.get(t):
                    i.append(tag[t])
        elif what == "trigger":
            for tag in src.get('containerVersion').get('tag'):
                if tag.get(tr):
                    i.append(tag[tr])
        elif what == "variable":
            for variable in src.get('containerVersion').get('variable'):
                if variable.get(v):
                    i.append(variable[v])

    else:
        i = src

    def deep_extract(ls):
        if type(ls) is list:
            for ids in ls:
                if not o.__contains__(ids):
                    deep_extract(ids)
        else:
            if not o.__contains__(ls):
                o.append(ls)

        return o

    for tr in i:
        deep_extract(tr)

    return o


def info(src: dict):
    if not src.get('containerVersion'):
        err = "sparrow error: GTM configuration file is not valid"
        print(err)
        exit(1)
    else:
        tags = len(extract_info(src, what='tag'))
        triggers = len(extract_info(src, what='trigger'))
        variables = len(extract_info(src, what='variable'))
        output = f"""
-----------------------------------------------------------
|                   General container info                |
-----------------------------------------------------------

Export-time:                {src['exportTime']}
Name:                       {src['containerVersion']['container'].get('name')}
Domain:                     {src['containerVersion']['container'].get('domainName')}
Public-id:                  {src['containerVersion']['container'].get('publicId')}
Container-id:               {src['containerVersion']['container'].get('containerId')}
Context:                    {src['containerVersion']['container'].get('usageContext')}
#Tags:                      {tags}
#Triggers:                  {triggers}
#Variables:                 {variables}

    """
        return output


def general_help(gray=False):
    if not gray:
        general_cli_help = """
Usage: sparrow [globals] <command> 

Globals:
  -v, --version                                 output the version number
  -h, --help                                    output usage information

Commands:
  extract <file.json> <input.json>              extract configuration file from source
  info <file.json>                              print information about the container


Examples

sparrow extract configuration.json target.json
    output -> output.json should be created in the current working directory

sparrow info configuration.json
    output -> Container related info

"""
        return general_cli_help
    else:
        general_cli_help = """
Usage: sparrow [globals] <command> 

Globals:
  -v, --version                                 output the version number
  -h, --help                                    output usage information

Commands:
  extract <file.json> <input.json> [flags]      extract configuration file from source
  info <file.json>                              print information about the container
  

Examples

sparrow extract configuration.json target.json
    output -> output.json should be created in the current working directory

sparrow info configuration.json
    output -> Container related info

        """
        return general_cli_help


def command_not_found(cmd):
    err = f"""sparrow command not found {cmd}\n"""
    return err



from setuptools import setup
import setuptools
from sparrow.src import __version__

setup(
    name='sparrow-gtm-cli',
    packages=setuptools.find_packages(),
    version=__version__,
    description='Google GTM Wrapper',
    license='GPLv3',
    author='Fady F Salem',
    author_email='ffaried@talabat.com',
    download_url='https://github.com/fadyfaried/sparrow-gtm-cli',
    keywords=['gtm', 'sparrow-cli'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Natural Language :: English',
    ],
    entry_points={
        'console_scripts': [
            'sparrow = sparrow.cli:main',
        ],
    }
)
